#Packages and indexes all images in a given folder to
#a single binary (.bin) and an index (.inc) file, respectively.
#This is for the CSE project ScoggerCSE (ScoggerC)

import sys
import getopt
import os
import time
from PIL import Image
from math import fabs
from math import sqrt
from ast import literal_eval

#-----------------------------------------------------------------------------
def readFile(file):
        a = []
        f = open(file,'rb')
        b = f.read(1)
        while b!=b'':
                a.append(ord(b))
                b = f.read(1)
        f.close()
        return a
 
def writeFile(file,a):
        f = open(file,'wb+')
        f.write(bytearray(a))
        f.close()
        
def appendFile(file,a):
        f = open(file,'ab')
        f.write(bytearray(a))
        f.close()
        
def xlc2rgb(xlc):
    r8 = ((((xlc>>2)&0x3E)|((xlc>>7)&1))/63.0)*255.0
    g8 = ((((xlc<<3)&0x38)|((xlc>>5)&7))/63.0)*255.0
    b8 = ((((xlc<<1)&0x3E)|((xlc>>4)&1))/63.0)*255.0
    return (r8,g8,b8)
    
def rgb2xlc(rgb):
    if isinstance(rgb,tuple):
        ir = rgb[0]
        ig = rgb[1]
        ib = rgb[2]
    else:
        print "Blah. Gimmie a tuple."
        sys.exit(2)
    pass
    dist = 9999.0
    cur_xlc = 0
    for i in range(256):
        tdist = sqrt(fabs(((((i>>2)&0x3E)|((i>>7)&1))/63.0)*255.0-ir)**2+fabs(((((i<<3)&0x38)|((i>>5)&7))/63.0)*255.0-ig)**2+fabs(((((i<<1)&0x3E)|((i>>4)&1))/63.0)*255.0-ib)**2)
        if tdist<dist:
            dist = tdist
            cur_xlc = i
    return cur_xlc
    
def rgb2xlc2(rgb):
    #G5:7 to 0:2
    #B6:7 to 3:4
    #R5:7 to 5:7
    a = (rgb[1]>>5)&0b00000111
    a = a | ((rgb[2]>>3)&0b00011000)
    a = a | (rgb[0]&0b11100000)
    return a
    
def img2cse(imgfname):
    global infpath
    fn = os.path.normpath(infpath + "\\" + imgfname)
    ifn = Image.open(fn)
    if ifn.mode != "RGB":
        iar = list(ifn.convert("RGB").getdata())
    else:
        iar = list(ifn.getdata())
    oar = []
    w = ifn.size[0]
    h = ifn.size[1]
    for x in range(w):
        for y in range(h):
            oar.append(rgb2xlc(iar[(y*w)+x]))
    return (w,h,oar)
    
#-----------------------------------------------------------------------------
#usage: imgpak.py <imgsPath> <outfileName(no ext)> [idxLblPrefix] 
#
#
#

if len(sys.argv)<3:
  print "Err: Not enough arguments passed!"
  sys.exit(2)
infpath = os.path.normpath(sys.argv[1])
outfname = sys.argv[2]
if os.path.isdir(infpath)==False:
    print("Error: Directory path does not exist: " + infpath)
    sys.exit(2)
    
try:
  lblprefix = sys.argv[3]
except:
  lblprefix = "" 

i = os.listdir(infpath)
img_namelist = [ fi for fi in i if (os.path.splitext(fi)[1].lower() in (".bmp",".png",".jpg"))]
print "Processing images..."

data_array = []
index_array = []

for imgname in img_namelist:
  print("Processing "+imgname+".\r"),
  #returns (w,h,array)
  t = img2cse(imgname)
  d = t[0]*t[1]
  #write 2-tuples of (lblname,index)
  index_array.append( (lblprefix+os.path.splitext(imgname)[0],len(data_array)) )
  #direct out: .db height,width,sizeL,sizeH,[array]
  if len(data_array)>65535:
    print "Err: Input gfx hives not allowed to be larger than 64KB"
    sys.exit(2)
  data_array.extend( [ t[1],t[0],d%256,d//256] )
  data_array.extend( t[2] )
  
print "\nOutputting files..."
writeFile(outfname+".bin",data_array)
writeFile(outfname+".inc",['\n'])
for i in index_array:
  t = i[0] + " .equ " + str(i[1]) + "\r\n"
  appendFile(outfname+".inc",t)
  
print "Files written. Array size: "+str(len(index_array)) + " elements, " +str(len(data_array)) + " bytes."


















